#ifndef _HERMIT_ARCH_RCM2_PINMUX_H_
#define _HERMIT_ARCH_RCM2_PINMUX_H_

#include <arch/system.h>

/* LSI Multiplexed Pin Setting Mask Register */
#define PMMR		0xe6060000
/* GPIO/peripheral function select register */
#define GPSR(gp) \
	((gp == 7) ? 0xe6060074 : 0xe6060004 + (gp * 4))
/* Peripheral function select register */
#define IPSR(ip) \
	((ip == 16) ? 0xe6060160 : 0xe6060020 + (ip * 4))
/* Module select register */
#define MOD_SEL(mod) \
	(0xe6060090 + (mod * 4))
/* LSI pin pull-up control register */
#define PUPR(pu) (0xe6060100 + (pu * 4))
/* SD control register */
#define IOCTRL0		0xe6060060
#define IOCTRL1		0xe6060064
/* TDSEL control register */
#define IOCTRL4		0xe6060084
#define IOCTRL5		0xe6060088
/* POC control register */
#define IOCTRL6		0xe606008c
/* IICDVFS and TDBGIO cell control register */
#define IOCTRL7		0xe6060070
/* DDR3 General Port IO Enable register */
#define DDR3GPEN	0xe6060240
/* DDR3 General Port Oputput Enable register */
#define DDR3GPOE	0xe6060244
/* DDR3 General Port Oputput Data register */
#define DDR3GPOD	0xe6060248
/* DDR3 General Port Input Data register */
#define DDR3GPID	0xe606024c


#define _shiftl(val, mask, shift) (((val) & (mask)) << (shift))
#define _shiftr(val, mask, shift) (((val) >> (shift)) & (mask))

#define _GP_IDNEX_MASK	0x07
#define _GP_IDNEX_SHIFT	5
#define _GP_SHIFT_MASK	0x1f
#define _GP_SHIFT_SHIFT	0
#define _GP(i, s) (_shiftl(i, _GP_IDNEX_MASK, _GP_IDNEX_SHIFT) | \
		   _shiftl(s, _GP_SHIFT_MASK, _GP_SHIFT_SHIFT))

#define _IP_IDNEX_MASK	0x1f
#define _IP_IDNEX_SHIFT	8
#define _IP_MASK_MASK	0x7
#define _IP_MASK_SHIFT	5
#define _IP_SHIFT_MASK	0x1f
#define _IP_SHIFT_SHIFT	0
#define _IP(i, m, s) (_shiftl(i, _IP_IDNEX_MASK, _IP_IDNEX_SHIFT) | \
		      _shiftl(m, _IP_MASK_MASK, _IP_MASK_SHIFT) |   \
		      _shiftl(s, _IP_SHIFT_MASK, _IP_SHIFT_SHIFT))

#define _PU_IDNEX_MASK	0x07
#define _PU_IDNEX_SHIFT	5
#define _PU_SHIFT_MASK	0x1f
#define _PU_SHIFT_SHIFT	0
#define _PU(i, s) (_shiftl(i, _PU_IDNEX_MASK, _PU_IDNEX_SHIFT) | \
		   _shiftl(s, _PU_SHIFT_MASK, _PU_SHIFT_SHIFT))

#define _PIN_GP_MASK	0xff
#define _PIN_GP_SHIFT	24
#define _PIN_IP_MASK	0xffff
#define _PIN_IP_SHIFT	8
#define _PIN_PU_MASK	0xff
#define _PIN_PU_SHIFT	0
#define _PIN_INFO(gp, ip, pu) (_shiftl(gp, _PIN_GP_MASK, _PIN_GP_SHIFT) | \
			       _shiftl(ip, _PIN_IP_MASK, _PIN_IP_SHIFT) | \
			       _shiftl(pu, _PIN_PU_MASK, _PIN_PU_SHIFT))

#define _GP_INVALID	(0xff)
#define _IP_INVALID	(0xffff)
#define _PU_INVALID	(0xff)

/*
 * RCM2 pins
 * =========
 *            (31....24)(23....16)(15.....8)(7......0)
 *       PIN: [   GP   ][        IP        ][   PU   ]
 *
 *            (7....5)(4........0)
 *        GP: [Index ][   Shift  ]
 *            (15..13)(12.......8)(7....5)(4........0)
 *        IP: [Unuse ][  Index   ][ Mask ][   Shift  ]
 *            (7....5)(4........0)
 *        PU: [Index ][   Shift  ]
 *
 * We use _PIN_INFO, _GP, _IP and _PU in a definition.
 */

/* pin: GP-0 */
#define PIN_D0			_PIN_INFO(_GP(0,0),_IP(0,1,0),_PU(0,6))
#define PIN_D1			_PIN_INFO(_GP(0,1),_IP(0,1,1),_PU(0,7))
#define PIN_D2			_PIN_INFO(_GP(0,2),_IP(0,1,2),_PU(0,8))
#define PIN_D3			_PIN_INFO(_GP(0,3),_IP(0,1,3),_PU(0,9))
#define PIN_D4			_PIN_INFO(_GP(0,4),_IP(0,1,4),_PU(0,10))
#define PIN_D5			_PIN_INFO(_GP(0,5),_IP(0,1,5),_PU(0,11))
#define PIN_D6			_PIN_INFO(_GP(0,6),_IP(0,1,6),_PU(0,12))
#define PIN_D7			_PIN_INFO(_GP(0,7),_IP(0,1,7),_PU(0,13))
#define PIN_D8			_PIN_INFO(_GP(0,8),_IP(0,1,8),_PU(0,14))
#define PIN_D9			_PIN_INFO(_GP(0,9),_IP(0,1,9),_PU(0,15))
#define PIN_D10			_PIN_INFO(_GP(0,10),_IP(0,1,10),_PU(0,16))
#define PIN_D11			_PIN_INFO(_GP(0,11),_IP(0,1,11),_PU(0,17))
#define PIN_D12			_PIN_INFO(_GP(0,12),_IP(0,1,12),_PU(0,18))
#define PIN_D13			_PIN_INFO(_GP(0,13),_IP(0,1,13),_PU(0,19))
#define PIN_D14			_PIN_INFO(_GP(0,14),_IP(0,1,14),_PU(0,20))
#define PIN_D15			_PIN_INFO(_GP(0,15),_IP(0,1,15),_PU(0,21))
#define PIN_A0			_PIN_INFO(_GP(0,16),_IP(0,7,16),_PU(0,22))
#define PIN_A1			_PIN_INFO(_GP(0,17),_IP(0,3,19),_PU(0,23))
#define PIN_A2			_PIN_INFO(_GP(0,18),_IP(0,3,21),_PU(0,24))
#define PIN_A3			_PIN_INFO(_GP(0,19),_IP(0,3,23),_PU(0,25))
#define PIN_A4			_PIN_INFO(_GP(0,20),_IP(0,3,25),_PU(0,26))
#define PIN_A5			_PIN_INFO(_GP(0,21),_IP(0,3,27),_PU(0,27))
#define PIN_A6			_PIN_INFO(_GP(0,22),_IP(0,3,29),_PU(0,28))
#define PIN_A7			_PIN_INFO(_GP(0,23),_IP(1,3,0),_PU(0,29))
#define PIN_A8			_PIN_INFO(_GP(0,24),_IP(1,3,2),_PU(0,30))
#define PIN_A9			_PIN_INFO(_GP(0,25),_IP(1,3,4),_PU(0,31))
#define PIN_A10			_PIN_INFO(_GP(0,26),_IP(1,3,6),_PU(1,0))
#define PIN_A11			_PIN_INFO(_GP(0,27),_IP(1,7,8),_PU(1,1))
#define PIN_A12			_PIN_INFO(_GP(0,28),_IP(1,7,11),_PU(1,2))
#define PIN_A13			_PIN_INFO(_GP(0,29),_IP(1,7,14),_PU(1,3))
#define PIN_A14			_PIN_INFO(_GP(0,30),_IP(1,7,17),_PU(1,4))
#define PIN_A15			_PIN_INFO(_GP(0,31),_IP(1,7,20),_PU(1,5))

/* pin: GP-1 */
#define PIN_A16			_PIN_INFO(_GP(1,0),_IP(1,7,23),_PU(1,6))
#define PIN_A17			_PIN_INFO(_GP(1,1),_IP(1,7,26),_PU(1,7))
#define PIN_A18			_PIN_INFO(_GP(1,2),_IP(1,7,29),_PU(1,8))
#define PIN_A19			_PIN_INFO(_GP(1,3),_IP(2,7,0),_PU(1,9))
#define PIN_A20			_PIN_INFO(_GP(1,4),_IP(2,3,3),_PU(0,0))
#define PIN_A21			_PIN_INFO(_GP(1,5),_IP(2,3,5),_PU(0,1))
#define PIN_A22			_PIN_INFO(_GP(1,6),_IP(2,7,7),_PU(0,2))
#define PIN_A23			_PIN_INFO(_GP(1,7),_IP(2,7,10),_PU(0,3))
#define PIN_A24			_PIN_INFO(_GP(1,8),_IP(2,7,13),_PU(0,4))
#define PIN_A25			_PIN_INFO(_GP(1,9),_IP(2,7,16),_PU(7,18))
#define PIN_CS0_N		_PIN_INFO(_GP(1,10),_IP(2,3,19),_PU(7,20))
#define PIN_CS1_N		_PIN_INFO(_GP(1,11),_IP(2,3,21),_PU_INVALID)
#define PIN_EX_CS0_N		_PIN_INFO(_GP(1,12),_IP_INVALID,_PU(1,15))
#define PIN_EX_CS1_N		_PIN_INFO(_GP(1,13),_IP(2,3,23),_PU(1,16))
#define PIN_EX_CS2_N		_PIN_INFO(_GP(1,14),_IP(2,3,25),_PU(1,17))
#define PIN_EX_CS3_N		_PIN_INFO(_GP(1,15),_IP(2,7,27),_PU(1,18))
#define PIN_EX_CS4_N		_PIN_INFO(_GP(1,16),_IP(3,7,0),_PU(1,19))
#define PIN_EX_CS5_N		_PIN_INFO(_GP(1,17),_IP(3,7,3),_PU(1,20))
#define PIN_BS_N		_PIN_INFO(_GP(1,18),_IP(3,7,6),_PU(1,21))
#define PIN_RD_N		_PIN_INFO(_GP(1,19),_IP_INVALID,_PU(1,22))
#define PIN_RD_WR_N		_PIN_INFO(_GP(1,20),_IP(3,7,9),_PU(1,23))
#define PIN_WE0_N		_PIN_INFO(_GP(1,21),_IP(3,3,12),_PU(1,24))
#define PIN_WE1_N		_PIN_INFO(_GP(1,22),_IP(3,3,14),_PU(1,25))
#define PIN_EX_WAIT0		_PIN_INFO(_GP(1,23),_IP(3,3,16),_PU(1,26))
#define PIN_DREQ0		_PIN_INFO(_GP(1,24),_IP(3,3,18),_PU(1,27))
#define PIN_DACK0		_PIN_INFO(_GP(1,25),_IP(3,3,20),_PU(1,28))

/* pin: GP-2 */
#define PIN_SSI_SCK0129		_PIN_INFO(_GP(2,0),_IP(3,7,25),_PU(1,30))
#define PIN_SSI_WS0129		_PIN_INFO(_GP(2,1),_IP(3,7,28),_PU(1,31))
#define PIN_SSI_SDATA0		_PIN_INFO(_GP(2,2),_IP(4,3,0),_PU(2,0))
#define PIN_SSI_SCK1		_PIN_INFO(_GP(2,3),_IP(4,7,2),_PU(2,1))
#define PIN_SSI_WS1		_PIN_INFO(_GP(2,4),_IP(4,7,5),_PU(2,2))
#define PIN_SSI_SDATA1		_PIN_INFO(_GP(2,5),_IP(4,3,8),_PU(2,3))
#define PIN_SSI_SCK2		_PIN_INFO(_GP(2,6),_IP(4,7,10),_PU(2,4))
#define PIN_SSI_WS2		_PIN_INFO(_GP(2,7),_IP(4,7,13),_PU(2,5))
#define PIN_SSI_SDATA2		_PIN_INFO(_GP(2,8),_IP(4,7,16),_PU(2,6))
#define PIN_SSI_SCK34		_PIN_INFO(_GP(2,9),_IP(4,1,19),_PU(2,7))
#define PIN_SSI_WS34		_PIN_INFO(_GP(2,10),_IP(4,1,20),_PU(2,8))
#define PIN_SSI_SDATA3		_PIN_INFO(_GP(2,11),_IP(4,1,21),_PU(2,9))
#define PIN_SSI_SCK4		_PIN_INFO(_GP(2,12),_IP(4,3,22),_PU(2,10))
#define PIN_SSI_WS4		_PIN_INFO(_GP(2,13),_IP(4,3,24),_PU(2,11))
#define PIN_SSI_SDATA4		_PIN_INFO(_GP(2,14),_IP(4,3,26),_PU(2,12))
#define PIN_SSI_SCK5		_PIN_INFO(_GP(2,15),_IP(4,7,28),_PU(2,13))
#define PIN_SSI_WS5		_PIN_INFO(_GP(2,16),_IP(5,7,0),_PU(2,14))
#define PIN_SSI_SDATA5		_PIN_INFO(_GP(2,17),_IP(5,7,3),_PU(2,15))
#define PIN_SSI_SCK6		_PIN_INFO(_GP(2,18),_IP(5,7,6),_PU(2,16))
#define PIN_SSI_WS6		_PIN_INFO(_GP(2,19),_IP(5,7,9),_PU(2,17))
#define PIN_SSI_SDATA6		_PIN_INFO(_GP(2,20),_IP(5,7,12),_PU(2,18))
#define PIN_SSI_SCK78		_PIN_INFO(_GP(2,21),_IP(5,3,15),_PU(2,19))
#define PIN_SSI_WS78		_PIN_INFO(_GP(2,22),_IP(5,7,17),_PU(2,20))
#define PIN_SSI_SDATA7		_PIN_INFO(_GP(2,23),_IP(5,3,20),_PU(2,21))
#define PIN_SSI_SDATA8		_PIN_INFO(_GP(2,24),_IP(5,3,22),_PU(2,22))
#define PIN_SSI_SCK9		_PIN_INFO(_GP(2,25),_IP(5,3,24),_PU(2,23))
#define PIN_SSI_WS9		_PIN_INFO(_GP(2,26),_IP(5,7,26),_PU(2,24))
#define PIN_SSI_SDATA9		_PIN_INFO(_GP(2,27),_IP(5,7,29),_PU(2,25))
#define PIN_AUDIO_CLKA		_PIN_INFO(_GP(2,28),_IP_INVALID,_PU(2,26))
#define PIN_AUDIO_CLKB		_PIN_INFO(_GP(2,29),_IP(6,7,0),_PU(2,27))
#define PIN_AUDIO_CLKC		_PIN_INFO(_GP(2,30),_IP(6,7,3),_PU(2,28))
#define PIN_AUDIO_CLKOUT	_PIN_INFO(_GP(2,31),_IP(6,3,6),_PU(2,29))

/* pin: GP-3 */
#define PIN_DU1_DR0		_PIN_INFO(_GP(3,0),_IP(7,7,3),_PU(3,8))
#define PIN_DU1_DR1		_PIN_INFO(_GP(3,1),_IP(7,7,6),_PU(3,9))
#define PIN_DU1_DR2		_PIN_INFO(_GP(3,2),_IP(7,3,9),_PU(3,10))
#define PIN_DU1_DR3		_PIN_INFO(_GP(3,3),_IP(7,3,11),_PU(3,11))
#define PIN_DU1_DR4		_PIN_INFO(_GP(3,4),_IP(7,3,13),_PU(3,12))
#define PIN_DU1_DR5		_PIN_INFO(_GP(3,5),_IP(7,3,15),_PU(3,13))
#define PIN_DU1_DR6		_PIN_INFO(_GP(3,6),_IP(7,3,17),_PU(3,14))
#define PIN_DU1_DR7		_PIN_INFO(_GP(3,7),_IP(7,3,19),_PU(3,15))
#define PIN_DU1_DG0		_PIN_INFO(_GP(3,8),_IP(7,7,21),_PU(3,16))
#define PIN_DU1_DG1		_PIN_INFO(_GP(3,9),_IP(7,7,24),_PU(3,17))
#define PIN_DU1_DG2		_PIN_INFO(_GP(3,10),_IP(7,7,27),_PU(3,18))
#define PIN_DU1_DG3		_PIN_INFO(_GP(3,11),_IP(8,7,0),_PU(3,19))
#define PIN_DU1_DG4		_PIN_INFO(_GP(3,12),_IP(8,7,3),_PU(3,20))
#define PIN_DU1_DG5		_PIN_INFO(_GP(3,13),_IP(8,7,6),_PU(3,21))
#define PIN_DU1_DG6		_PIN_INFO(_GP(3,14),_IP(8,7,9),_PU(3,22))
#define PIN_DU1_DG7		_PIN_INFO(_GP(3,15),_IP(8,7,12),_PU(3,23))
#define PIN_DU1_DB0		_PIN_INFO(_GP(3,16),_IP(8,7,15),_PU(3,24))
#define PIN_DU1_DB1		_PIN_INFO(_GP(3,17),_IP(8,7,18),_PU(3,25))
#define PIN_DU1_DB2		_PIN_INFO(_GP(3,18),_IP(8,7,21),_PU(3,26))
#define PIN_DU1_DB3		_PIN_INFO(_GP(3,19),_IP(8,3,24),_PU(3,27))
#define PIN_DU1_DB4		_PIN_INFO(_GP(3,20),_IP(8,3,26),_PU(3,28))
#define PIN_DU1_DB5		_PIN_INFO(_GP(3,21),_IP(8,7,28),_PU(3,29))
#define PIN_DU1_DB6		_PIN_INFO(_GP(3,22),_IP(9,7,0),_PU(3,30))
#define PIN_DU1_DB7		_PIN_INFO(_GP(3,23),_IP(9,7,3),_PU(3,31))
#define PIN_DU1_DOTCLKIN	_PIN_INFO(_GP(3,24),_IP(9,1,6),_PU(4,0))
#define PIN_DU1_DOTCLKOUT0	_PIN_INFO(_GP(3,25),_IP(9,1,7),_PU(4,1))
#define PIN_DU1_DOTCLKOUT1	_PIN_INFO(_GP(3,26),_IP(9,7,8),_PU(4,2))
#define PIN_DU1_EXHSYNC		_PIN_INFO(_GP(3,27),_IP(9,1,11),_PU(4,3))
#define PIN_DU1_EXVSYNC		_PIN_INFO(_GP(3,28),_IP(9,1,12),_PU(4,4))
#define PIN_DU1_EXODDF		_PIN_INFO(_GP(3,29),_IP(9,7,13),_PU(4,5))
#define PIN_DU1_DISP		_PIN_INFO(_GP(3,30),_IP(9,1,16),_PU(4,6))
#define PIN_DU1_CDE		_PIN_INFO(_GP(3,31),_IP(9,3,17),_PU(4,7))

/* pin: GP-4 */
#define PIN_VI0_CLK		_PIN_INFO(_GP(4,0),_IP_INVALID,_PU(4,8))
#define PIN_VI0_CLKENB		_PIN_INFO(_GP(4,1),_IP(9,3,19),_PU(4,9))
#define PIN_VI0_FIELD		_PIN_INFO(_GP(4,2),_IP(9,3,21),_PU(4,10))
#define PIN_VI0_HSYNC_N		_PIN_INFO(_GP(4,3),_IP(9,3,23),_PU(4,11))
#define PIN_VI0_VSYNC_N		_PIN_INFO(_GP(4,4),_IP(9,3,25),_PU(4,12))
#define PIN_VI0_DATA0		_PIN_INFO(_GP(4,5),_IP_INVALID,_PU(4,13))
#define PIN_VI0_DATA1		_PIN_INFO(_GP(4,6),_IP_INVALID,_PU(4,14))
#define PIN_VI0_DATA2		_PIN_INFO(_GP(4,7),_IP_INVALID,_PU(4,15))
#define PIN_VI0_DATA3		_PIN_INFO(_GP(4,8),_IP(9,3,27),_PU(4,16))
#define PIN_VI0_DATA4		_PIN_INFO(_GP(4,9),_IP_INVALID,_PU(4,17))
#define PIN_VI0_DATA5		_PIN_INFO(_GP(4,10),_IP_INVALID,_PU(4,18))
#define PIN_VI0_DATA6		_PIN_INFO(_GP(4,11),_IP_INVALID,_PU(4,19))
#define PIN_VI0_DATA7		_PIN_INFO(_GP(4,12),_IP_INVALID,_PU(4,20))
#define PIN_VI0_G0		_PIN_INFO(_GP(4,13),_IP(9,7,29),_PU(4,21))
#define PIN_VI0_G1		_PIN_INFO(_GP(4,14),_IP(10,7,0),_PU(4,22))
#define PIN_VI0_G2		_PIN_INFO(_GP(4,15),_IP(10,7,3),_PU(4,23))
#define PIN_VI0_G3		_PIN_INFO(_GP(4,16),_IP(10,7,6),_PU(4,24))
#define PIN_VI0_G4		_PIN_INFO(_GP(4,17),_IP(10,7,9),_PU(4,25))
#define PIN_VI0_G5		_PIN_INFO(_GP(4,18),_IP(10,7,12),_PU(4,26))
#define PIN_VI0_G6		_PIN_INFO(_GP(4,19),_IP(10,3,15),_PU(4,27))
#define PIN_VI0_G7		_PIN_INFO(_GP(4,20),_IP(10,3,17),_PU(4,28))
#define PIN_VI0_R0		_PIN_INFO(_GP(4,21),_IP(10,7,19),_PU(4,29))
#define PIN_VI0_R1		_PIN_INFO(_GP(4,22),_IP(10,7,22),_PU(4,30))
#define PIN_VI0_R2		_PIN_INFO(_GP(4,23),_IP(10,3,25),_PU(4,31))
#define PIN_VI0_R3		_PIN_INFO(_GP(4,24),_IP(10,3,27),_PU(5,0))
#define PIN_VI0_R4		_PIN_INFO(_GP(4,25),_IP(10,7,29),_PU(5,1))
#define PIN_VI0_R5		_PIN_INFO(_GP(4,26),_IP(11,7,0),_PU(5,2))
#define PIN_VI0_R6		_PIN_INFO(_GP(4,27),_IP(11,7,3),_PU(5,3))
#define PIN_VI0_R7		_PIN_INFO(_GP(4,28),_IP(11,7,6),_PU(5,4))
#define PIN_SIM0_RST		_PIN_INFO(_GP(4,29),_IP(15,3,0),_PU(7,2))
#define PIN_SIM0_CLK		_PIN_INFO(_GP(4,30),_IP(15,3,2),_PU(7,3))
#define PIN_SIM0_D		_PIN_INFO(_GP(4,31),_IP(15,3,4),_PU(7,4))

/* pin: GP-5 */
#define PIN_VI1_HSYNC_N		_PIN_INFO(_GP(5,0),_IP(11,7,9),_PU(5,5))
#define PIN_VI1_VSYNC_N		_PIN_INFO(_GP(5,1),_IP(11,7,12),_PU(5,6))
#define PIN_VI1_CLKENB		_PIN_INFO(_GP(5,2),_IP(11,3,15),_PU(5,7))
#define PIN_VI1_FIELD		_PIN_INFO(_GP(5,3),_IP(11,3,17),_PU(5,8))
#define PIN_VI1_CLK		_PIN_INFO(_GP(5,4),_IP(11,1,19),_PU(5,9))
#define PIN_VI1_DATA0		_PIN_INFO(_GP(5,5),_IP(11,1,20),_PU(5,10))
#define PIN_VI1_DATA1		_PIN_INFO(_GP(5,6),_IP(11,1,21),_PU(5,11))
#define PIN_VI1_DATA2		_PIN_INFO(_GP(5,7),_IP(11,1,22),_PU(5,12))
#define PIN_VI1_DATA3		_PIN_INFO(_GP(5,8),_IP(11,1,23),_PU(5,13))
#define PIN_VI1_DATA4		_PIN_INFO(_GP(5,9),_IP(11,1,24),_PU(5,14))
#define PIN_VI1_DATA5		_PIN_INFO(_GP(5,10),_IP(11,1,25),_PU(5,15))
#define PIN_VI1_DATA6		_PIN_INFO(_GP(5,11),_IP(11,1,26),_PU(5,16))
#define PIN_VI1_DATA7		_PIN_INFO(_GP(5,12),_IP(11,1,27),_PU(5,17))
#define PIN_ETH_MDIO		_PIN_INFO(_GP(5,13),_IP(11,3,28),_PU(5,18))
#define PIN_ETH_CRS_DV		_PIN_INFO(_GP(5,14),_IP(11,3,30),_PU(5,19))
#define PIN_ETH_RX_ER		_PIN_INFO(_GP(5,15),_IP(12,3,0),_PU(5,20))
#define PIN_ETH_RXD0		_PIN_INFO(_GP(5,16),_IP(12,3,2),_PU(5,21))
#define PIN_ETH_RXD1		_PIN_INFO(_GP(5,17),_IP(12,7,4),_PU(5,22))
#define PIN_ETH_LINK		_PIN_INFO(_GP(5,18),_IP(12,7,7),_PU(5,23))
#define PIN_ETH_REFCLK		_PIN_INFO(_GP(5,19),_IP(12,7,10),_PU(5,24))
#define PIN_ETH_TXD1		_PIN_INFO(_GP(5,20),_IP(12,7,13),_PU(5,25))
#define PIN_ETH_TX_EN		_PIN_INFO(_GP(5,21),_IP(12,3,16),_PU(5,26))
#define PIN_ETH_MAGIC		_PIN_INFO(_GP(5,22),_IP(12,3,18),_PU(5,27))
#define PIN_ETH_TXD0		_PIN_INFO(_GP(5,23),_IP(12,3,20),_PU(5,28))
#define PIN_ETH_MDC		_PIN_INFO(_GP(5,24),_IP(12,3,22),_PU(5,29))
#define PIN_STP_IVCXO27_0	_PIN_INFO(_GP(5,25),_IP(12,7,24),_PU(5,30))
#define PIN_STP_ISCLK_0		_PIN_INFO(_GP(5,26),_IP(12,7,27),_PU(5,31))
#define PIN_STP_ISD_0		_PIN_INFO(_GP(5,27),_IP(13,7,0),_PU(6,0))
#define PIN_STP_ISEN_0		_PIN_INFO(_GP(5,28),_IP(13,3,3),_PU(6,1))
#define PIN_STP_ISSYNC_0	_PIN_INFO(_GP(5,29),_IP(13,3,5),_PU(6,2))
#define PIN_STP_OPWM_0		_PIN_INFO(_GP(5,30),_IP(13,7,7),_PU(6,3))
#define PIN_SPEEDIN		_PIN_INFO(_GP(5,31),_IP(3,7,22),_PU(1,29))

/* pin: GP-6 */
#define PIN_SD0_CLK		_PIN_INFO(_GP(6,0),_IP(13,1,10),_PU(6,4))
#define PIN_SD0_CMD		_PIN_INFO(_GP(6,1),_IP(13,1,11),_PU(6,5))
#define PIN_SD0_DATA0		_PIN_INFO(_GP(6,2),_IP(13,1,12),_PU(6,6))
#define PIN_SD0_DATA1		_PIN_INFO(_GP(6,3),_IP(13,1,13),_PU(6,7))
#define PIN_SD0_DATA2		_PIN_INFO(_GP(6,4),_IP(13,1,14),_PU(6,8))
#define PIN_SD0_DATA3		_PIN_INFO(_GP(6,5),_IP(13,1,15),_PU(6,9))
#define PIN_SD0_CD		_PIN_INFO(_GP(6,6),_IP(13,7,16),_PU(6,10))
#define PIN_SD0_WP		_PIN_INFO(_GP(6,7),_IP(13,7,19),_PU(6,11))
#define PIN_SD1_CLK		_PIN_INFO(_GP(6,8),_IP_INVALID,_PU(6,12))
#define PIN_SD1_CMD		_PIN_INFO(_GP(6,9),_IP(13,1,22),_PU(6,13))
#define PIN_SD1_DATA0		_PIN_INFO(_GP(6,10),_IP(13,3,23),_PU(6,14))
#define PIN_SD1_DATA1		_PIN_INFO(_GP(6,11),_IP(13,1,25),_PU(6,15))
#define PIN_SD1_DATA2		_PIN_INFO(_GP(6,12),_IP(13,1,26),_PU(6,16))
#define PIN_SD1_DATA3		_PIN_INFO(_GP(6,13),_IP(13,1,27),_PU(6,17))
#define PIN_SD1_CD		_PIN_INFO(_GP(6,14),_IP(13,7,28),_PU(6,18))
#define PIN_SD1_WP		_PIN_INFO(_GP(6,15),_IP(14,3,0),_PU(6,19))
#define PIN_SD2_CLK		_PIN_INFO(_GP(6,16),_IP(14,1,2),_PU(6,20))
#define PIN_SD2_CMD		_PIN_INFO(_GP(6,17),_IP(14,1,3),_PU(6,21))
#define PIN_SD2_DATA0		_PIN_INFO(_GP(6,18),_IP(14,1,4),_PU(6,22))
#define PIN_SD2_DATA1		_PIN_INFO(_GP(6,19),_IP(14,1,5),_PU(6,23))
#define PIN_SD2_DATA2		_PIN_INFO(_GP(6,20),_IP(14,1,6),_PU(6,24))
#define PIN_SD2_DATA3		_PIN_INFO(_GP(6,21),_IP(14,1,7),_PU(6,25))
#define PIN_SD2_CD		_PIN_INFO(_GP(6,22),_IP(14,7,8),_PU(6,26))
#define PIN_SD2_WP		_PIN_INFO(_GP(6,23),_IP(14,7,11),_PU(6,27))
#define PIN_MSIOF0_SCK		_PIN_INFO(_GP(6,24),_IP(14,7,14),_PU(6,28))
#define PIN_MSIOF0_SYNC		_PIN_INFO(_GP(6,25),_IP(14,7,17),_PU(6,29))
#define PIN_MSIOF0_TXD		_PIN_INFO(_GP(6,26),_IP(14,7,20),_PU(6,30))
#define PIN_MSIOF0_RXD		_PIN_INFO(_GP(6,27),_IP(14,7,23),_PU(6,31))
#define PIN_MSIOF0_SS1		_PIN_INFO(_GP(6,28),_IP(14,7,26),_PU(7,0))
#define PIN_MSIOF0_SS2		_PIN_INFO(_GP(6,29),_IP(14,7,29),_PU(7,1))
#define PIN_USB1_OVC		_PIN_INFO(_GP(6,30),_IP_INVALID,_PU(7,24))
#define PIN_DU0_DOTCLKIN	_PIN_INFO(_GP(6,31),_IP_INVALID,_PU(0,5))

/* pin: GP-7 */
#define PIN_HCTS0_N		_PIN_INFO(_GP(7,0),_IP(15,7,15),_PU(7,8))
#define PIN_HRTS0_N		_PIN_INFO(_GP(7,1),_IP(15,7,18),_PU(7,9))
#define PIN_HSCK0		_PIN_INFO(_GP(7,2),_IP(15,7,21),_PU(7,10))
#define PIN_HRX0		_PIN_INFO(_GP(7,3),_IP(15,7,24),_PU(7,11))
#define PIN_HTX0		_PIN_INFO(_GP(7,4),_IP(15,7,27),_PU(7,12))
#define PIN_HRX1		_PIN_INFO(_GP(7,5),_IP(16,7,0),_PU(7,13))
#define PIN_HTX1		_PIN_INFO(_GP(7,6),_IP(16,7,3),_PU(7,14))
#define PIN_HSCK1		_PIN_INFO(_GP(7,7),_IP(16,3,6),_PU_INVALID)
#define PIN_HCTS1_N		_PIN_INFO(_GP(7,8),_IP(16,3,8),_PU_INVALID)
#define PIN_HRTS1_N		_PIN_INFO(_GP(7,9),_IP(16,3,10),_PU_INVALID)
#define PIN_IRQ0		_PIN_INFO(_GP(7,10),_IP(6,3,8),_PU(2,30))
#define PIN_IRQ1		_PIN_INFO(_GP(7,11),_IP(6,3,10),_PU(2,31))
#define PIN_IRQ2		_PIN_INFO(_GP(7,12),_IP(6,3,12),_PU(3,0))
#define PIN_IRQ3		_PIN_INFO(_GP(7,13),_IP(6,3,14),_PU(3,1))
#define PIN_IRQ4		_PIN_INFO(_GP(7,14),_IP(6,7,16),_PU(3,2))
#define PIN_IRQ5		_PIN_INFO(_GP(7,15),_IP(6,3,19),_PU(3,3))
#define PIN_IRQ6		_PIN_INFO(_GP(7,16),_IP(6,7,21),_PU(3,4))
#define PIN_IRQ7		_PIN_INFO(_GP(7,17),_IP(6,7,24),_PU(3,5))
#define PIN_IRQ8		_PIN_INFO(_GP(7,18),_IP(6,7,27),_PU(3,6))
#define PIN_IRQ9		_PIN_INFO(_GP(7,19),_IP(7,7,0),_PU(3,7))
#define PIN_GPS_CLK		_PIN_INFO(_GP(7,20),_IP(15,7,6),_PU(7,5))
#define PIN_GPS_SIGN		_PIN_INFO(_GP(7,21),_IP(15,7,9),_PU(7,6))
#define PIN_GPS_MAG		_PIN_INFO(_GP(7,22),_IP(15,7,12),_PU(7,7))
#define PIN_USB0_PWEN		_PIN_INFO(_GP(7,23),_IP_INVALID,_PU(7,21))
#define PIN_USB0_OVC		_PIN_INFO(_GP(7,24),_IP_INVALID,_PU(7,22))
#define PIN_USB1_PWEN		_PIN_INFO(_GP(7,25),_IP_INVALID,_PU(7,23))

/* pin: other */
#define PIN_TRST_N		_PIN_INFO(_GP_INVALID,_IP_INVALID,_PU(1,10))
#define PIN_TCK			_PIN_INFO(_GP_INVALID,_IP_INVALID,_PU(1,11))
#define PIN_TMS			_PIN_INFO(_GP_INVALID,_IP_INVALID,_PU(1,12))
#define PIN_TDI			_PIN_INFO(_GP_INVALID,_IP_INVALID,_PU(1,13))
#define PIN_ASEBRK_N		_PIN_INFO(_GP_INVALID,_IP_INVALID,_PU(7,19))
#define PIN_AVS1		_PIN_INFO(_GP_INVALID,_IP_INVALID,_PU(7,25))
#define PIN_AVS2		_PIN_INFO(_GP_INVALID,_IP_INVALID,_PU(7,26))

typedef enum {
	GP_GPIO = 0,
	GP_FUNC = 1,
} gpsel;

typedef enum {
	IP_FUNC1 = 0,
	IP_FUNC2 = 1,
	IP_FUNC3 = 2,
	IP_FUNC4 = 3,
	IP_FUNC5 = 4,
	IP_FUNC6 = 5,
	IP_FUNC7 = 6,
	IP_OTHER = 7,
} ipsel;

typedef enum {
	PU_DIS = 0,
	PU_EN = 1,
} pusel;

#define _PIN2GP(pin)		(_shiftr(pin, _PIN_GP_MASK, _PIN_GP_SHIFT))
#define _PIN2IP(pin)		(_shiftr(pin, _PIN_IP_MASK, _PIN_IP_SHIFT))
#define _PIN2PU(pin)		(_shiftr(pin, _PIN_PU_MASK, _PIN_PU_SHIFT))

#define IS_VALID_GP(pin)	(_PIN2GP(pin) == _GP_INVALID ? 0 : 1)
#define IS_VALID_IP(pin)	(_PIN2IP(pin) == _IP_INVALID ? 0 : 1)
#define IS_VALID_PU(pin)	(_PIN2PU(pin) == _PU_INVALID ? 0 : 1)

/* GP restore macros */
#define _PIN2GPINDEX(pin)	(_shiftr(_PIN2GP(pin), _GP_IDNEX_MASK, \
					 _GP_IDNEX_SHIFT))
#define PIN2GPSR(pin)		(GPSR(_PIN2GPINDEX(pin)))

#define _PIN2GPSHIFT(pin)	(_shiftr(_PIN2GP(pin), _GP_SHIFT_MASK, \
					 _GP_SHIFT_SHIFT))
#define PIN2GPMASK(pin)		(1 << _PIN2GPSHIFT(pin))

/* IP restore macros */
#define _PIN2IPINDEX(pin)	(_shiftr(_PIN2IP(pin), _IP_IDNEX_MASK, \
					 _IP_IDNEX_SHIFT))
#define PIN2IPSR(pin)		(IPSR(_PIN2IPINDEX(pin)))

#define PIN2IPSHIFT(pin)	(_shiftr(_PIN2IP(pin), _IP_SHIFT_MASK, \
					 _IP_SHIFT_SHIFT))

#define _PIN2IPMASK(pin)	(_shiftr(_PIN2IP(pin), _IP_MASK_MASK, \
					 _IP_MASK_SHIFT))
#define PIN2IPMASK(pin)		(_PIN2IPMASK(pin) << PIN2IPSHIFT(pin))

/* PU restore macros */
#define _PIN2PUINDEX(pin)	(_shiftr(_PIN2PU(pin), _PU_IDNEX_MASK, \
					 _PU_IDNEX_SHIFT))
#define PIN2PUPR(pin)		(PUPR(_PIN2PUINDEX(pin)))

#define _PIN2PUSHIFT(pin)	(_shiftr(_PIN2PU(pin), _PU_SHIFT_MASK, \
					 _PU_SHIFT_SHIFT))
#define PIN2PUMASK(pin)		(1 << _PIN2PUSHIFT(pin))


#define _MOD_IDNEX_MASK		0x3
#define _MOD_IDNEX_SHIFT	8
#define _MOD_MASK_MASK		0x7
#define _MOD_MASK_SHIFT		5
#define _MOD_SHIFT_MASK		0x1f
#define _MOD_SHIFT_SHIFT	0
#define _MOD_INFO(i, m, s) (_shiftl(i, _MOD_IDNEX_MASK, _MOD_IDNEX_SHIFT) | \
			    _shiftl(m, _MOD_MASK_MASK, _MOD_MASK_SHIFT) | \
			    _shiftl(s, _MOD_SHIFT_MASK, _MOD_SHIFT_SHIFT))

/* mod_sel */
#define MOD_SCIF1	_MOD_INFO(0, 3, 29)
#define MOD_SCIFB	_MOD_INFO(0, 3, 27)
#define MOD_SCIFB2	_MOD_INFO(0, 1, 26)
#define MOD_SCIFB1	_MOD_INFO(0, 7, 22)
#define MOD_SCIFA1	_MOD_INFO(0, 3, 20)
#define MOD_SSI9	_MOD_INFO(0, 1, 19)
#define MOD_SCFA	_MOD_INFO(0, 1, 18)
#define MOD_QSP		_MOD_INFO(0, 1, 17)
#define MOD_SSI7	_MOD_INFO(0, 1, 16)
#define MOD_HSCIF1	_MOD_INFO(0, 7, 13)
#define MOD_IRD		_MOD_INFO(0, 3, 11)
#define MOD_VI1		_MOD_INFO(0, 3, 9)
#define MOD_TMU1	_MOD_INFO(0, 1, 6)
#define MOD_LBS		_MOD_INFO(0, 1, 4)
#define MOD_TSIF0	_MOD_INFO(0, 3, 2)
#define MOD_SOF0	_MOD_INFO(0, 3, 0)

/* mod_sel2 */
#define MOD_SCIF0	_MOD_INFO(1, 7, 29)
#define MOD_SCIF	_MOD_INFO(1, 1, 27)
#define MOD_CAN0	_MOD_INFO(1, 7, 24)
#define MOD_CAN1	_MOD_INFO(1, 3, 22)
#define MOD_SCIFA2	_MOD_INFO(1, 1, 20)
#define MOD_SCIF4	_MOD_INFO(1, 3, 18)
#define MOD_ADI		_MOD_INFO(1, 1, 16)
#define MOD_ADG		_MOD_INFO(1, 1, 15)
#define MOD_FM		_MOD_INFO(1, 7, 12)
#define MOD_SCIFA5	_MOD_INFO(1, 3, 10)
#define MOD_GPS		_MOD_INFO(1, 3, 7)
#define MOD_SCIFA4	_MOD_INFO(1, 3, 5)
#define MOD_SCIFA3	_MOD_INFO(1, 3, 3)
#define MOD_SIM		_MOD_INFO(1, 1, 2)
#define MOD_SSI8	_MOD_INFO(1, 1, 0)

/* mod_sel3 */
#define MOD_HSCIF2	_MOD_INFO(2, 3, 30)
#define MOD_CANCLK	_MOD_INFO(2, 3, 28)
#define MOD_IIC8	_MOD_INFO(2, 3, 26)
#define MOD_IIC7	_MOD_INFO(2, 3, 24)
#define MOD_IIC4	_MOD_INFO(2, 3, 22)
#define MOD_IIC3	_MOD_INFO(2, 3, 20)
#define MOD_SCIF3	_MOD_INFO(2, 3, 18)
#define MOD_IEB		_MOD_INFO(2, 3, 16)
#define MOD_MMC		_MOD_INFO(2, 1, 15)
#define MOD_SCIF5	_MOD_INFO(2, 1, 14)
#define MOD_IIC2	_MOD_INFO(2, 3, 10)
#define MOD_IIC1	_MOD_INFO(2, 7, 7)
#define MOD_IIC0	_MOD_INFO(2, 3, 5)

/* mod_sel4 */
#define MOD_SOF1	_MOD_INFO(3, 7, 29)
#define MOD_HSCIF0	_MOD_INFO(3, 3, 27)
#define MOD_DIS		_MOD_INFO(3, 3, 25)
#define MOD_FSN		_MOD_INFO(3, 1, 24)
#define MOD_RAD		_MOD_INFO(3, 1, 23)
#define MOD_RCN		_MOD_INFO(3, 1, 22)
#define MOD_RSP		_MOD_INFO(3, 1, 21)
#define MOD_SCIF2	_MOD_INFO(3, 7, 18)
#define MOD_SOF2	_MOD_INFO(3, 7, 11)
#define MOD_SPE		_MOD_INFO(3, 1, 10)
#define MOD_SSI1	_MOD_INFO(3, 1, 9)
#define MOD_SSI0	_MOD_INFO(3, 1, 8)
#define MOD_SSP		_MOD_INFO(3, 3, 6)

typedef enum {
	MOD_FUNC1 = 0,
	MOD_FUNC2 = 1,
	MOD_FUNC3 = 2,
	MOD_FUNC4 = 3,
	MOD_FUNC5 = 4,
	MOD_FUNC6 = 5,
} modfunc;

/* MOD restore macros */

#define _MOD2INDEX(mod)		(_shiftr(mod, _MOD_IDNEX_MASK, \
					 _MOD_IDNEX_SHIFT))
#define MOD2MOD_SEL(mod)	(MOD_SEL(_MOD2INDEX(mod)))

#define MOD2SHIFT(mod)		(_shiftr(mod, _MOD_SHIFT_MASK, \
					 _MOD_SHIFT_SHIFT))

#define _MOD2MASK(mod)		(_shiftr(mod, _MOD_MASK_MASK, \
					 _MOD_MASK_SHIFT))
#define MOD2MASK(mod)		(_MOD2MASK(mod) << MOD2SHIFT(mod))


void pinmux(u32 pin, gpsel gp, ipsel ip, pusel pu);
void modsel(u16 mod, modfunc sel);

#endif
